#import needed modules
import os
import re

#save the value that was entered in the knob "prerender_renderpath" - get the directory path of that value and add a slash
#if the user entered a file instead of a directory then "os.path.dirname" makes sure to just return this directory name in which the file is located
renderPath = os.path.dirname(nuke.thisNode()["prerender_renderpath"].getValue()) + "/"
#framepadding set to 4 digits, i.e. for example:
# render_%04d.tif
# render_####.tif
# render_1227.tif render_1228.tif render_1229.tif
padding = "%04d"
#format the entered file type. Dropdowns (pulldown choice) knobs always return a floating point number with their value depending on the line that was choosen - starting at 0.0
#for example if the user chose ".exr" (first entry) then the fileType's value returns "0.0". If on the other hand he chose ".png" then fileType's value will return "3.0".
#check this value and overwrite the variable "ext" with the according file type as a string
ext = nuke.thisNode()["fileType"].getValue()
if ext == 0.0:
    ext = "exr"
if ext == 1.0:
    ext = "dpx"
if ext == 2.0:
    ext = "tif"
if ext == 3.0:
    ext = "png"

#try to find the version number in the name of the current nuke script. This can be achieved with a regula expression.
try:
    curVersion = re.findall(r"_[Vv]\d*", os.path.basename(nuke.root().name()))[0]
    renderPath += curVersion+"/"
except:
	#if there is no version in the name of the current nukescript then initialize the variable "curVersion" with an empty string
    curVersion = ""

#main function "makePrerenders" which creates all prerender nodes. When calling this function the parameter 'node' will be the node to create a prerender from (in the following called the "assigned node") 
def makePrerender(node):
    #deselect all nodes
    nukescripts.clear_selection_recursive()
    #select only the assigned
    node["selected"].setValue(True)

	#save all nodes that are connected to the assigned node
    dependentNodes = node.dependent()  

	#set the write location, which is the renderpath (see above) and the node name itself
    writeLocation = renderPath+node.name()
    #if this directory doesn't exist then create ir
    if not os.path.isdir(writeLocation):
        os.makedirs(writeLocation)

    #write
    #create prerender write node
    
    #create write node, don't open its attributes in the properties bin
    write = nuke.createNode("Write",inpanel = False)
    #set the renderlocation in the file knob: writeLocation (see above) + "/" + nodename + curVersion + "_" + framepadding + "." + extension
    write["file"].setValue(writeLocation+"/{name}{version}_{padding}.{ext}".format(name = node.name(), version = curVersion, padding = padding, ext = ext))
    #give the write node the keyword "intermediate" in its knob "label"
    write["label"].setValue("_intermediate_")
    #set the according filetype
    write["file_type"].setValue(ext)
    #render all channels
    write["channels"].setValue("all")
    #rename the write node
    write.setName("intermediate_%s" % node.name())
    #position the write node
    write.setXpos(node.xpos())
    write.setYpos(node.ypos()+200)
    #connect the write node with the assigned node
    write.setInput(0,node)

    #switch
   	#create switch node, don't open its attributes in the properties bin
    switch = nuke.createNode("Switch",inpanel = False)
    #set its which knob to "1"
    switch["which"].setValue(1)
    #position the switch node
    switch.setXpos(write.xpos())
    switch.setYpos(write.ypos()+80)
    #give the switch node the keyword "intermediate" in its knob "label"
    switch["label"].setValue("_intermediate_\n(1)read prerenders")
    #connect the swith node with the above created write node
    switch.setInput(0,write)
    
    #read
    #create read node, don't open its attributes in the properties bin
    read = nuke.createNode("Read",inpanel = False)
    #set the readlocation in the file knob: same location which was set in the write node (see above)
    read["file"].setValue(writeLocation+"/{name}{version}_{padding}.{ext}".format(name = node.name(), version = curVersion, padding = padding, ext = ext))
    #first frame of footage is the first frame of the current nukescript
    read["first"].setValue(int(nuke.root()["first_frame"].getValue()))
    #origfirst frame of footage is the first frame of the current nukescript
    read["origfirst"].setValue(int(nuke.root()["first_frame"].getValue()))
    #last frame of footage is the last frame of the current nukescript
    read["last"].setValue(int(nuke.root()["last_frame"].getValue()))
    #origlast frame of footage is the last frame of the current nukescript
    read["origlast"].setValue(int(nuke.root()["last_frame"].getValue()))
    #position node
    read.setXpos(switch.xpos()-120)
    read.setYpos(write.ypos()+40)
    #give the read node the keyword "intermediate" in its knob "label"
    read["label"].setValue("_intermediate_")   
    #connect the above created switch node with the read node
    switch.setInput(1,read)

    #backdrop
    #create backdrop node, don't open its attributes in the properties bin
    bd = nuke.createNode("BackdropNode",inpanel = False)
    #position the backdrop node
    bd.setXpos(write.xpos()-150)
    bd.setYpos(write.ypos()-80)
    #set the backdrop's node width and height to cover all created prerender nodes
    bd["bdwidth"].setValue(300)
    bd["bdheight"].setValue(230)
    #bigger font size
    bd["note_font_size"].setValue(25)
    #give the backdrop node the keyword "intermediate" in its knob "label"
    bd["label"].setValue("_intermediate_%s"%node.name())
    #color the backdrop node purple
    bd["tile_color"].setValue(654977023)

	#connect all nodes that were connected to the assigned node with the prerender switch node
    for node in dependentNodes:
        node.setInput(0,switch)

#prerender loop
#save all selected nodes in variable "nodesToPrerender"
nodesToPrerender=nuke.selectedNodes()

#if version in current nukescript's name found
if curVersion != "":
	#if user entered a renderpath
    if renderPath != "" and renderPath != "/":
        #if user selected some nodes to prerender
        if len(nodesToPrerender)!=0:
        	#ask user if he renamed all nodes to prerender (optional but recomended to better identify the prerenders)
            if nuke.ask("Please give all selected nodes some reasonable names to create meaningful and recognizable prerender elements. Did you do that?"):
                #iterate every node to prerender
                for node in nodesToPrerender:
                	#if node to prerender is not the PrerenderManager itself or a viewer
                    if node.name() != "PrerenderManager" and node.Class() != "Viewer":
                        #call function makePrerender and assign node to prerender
                        makePrerender(node)
        #print message that no nodes to prerender were selected
        else:
            nuke.message("Please select some nodes to prerender. Please give the last node before the render a meaningful name to create meaningful prerender element names.")
    #print message that a render path wasn't entered
    else:
        nuke.message("Please set up a render path for the prerenders")
#print message that the current nukescript doesn't contain a version number
else:
    nuke.message("No version found. Please make sure that the script has a version.")



